/**
 * xmotd part,working in local file system.  Version 0.55
 *
 * Copyright (C) 2001  Olga Joloud (ojoloud@ee.ryerson.ca)
 * All Rights Reserved.
 *
 * v 0.55 2001/04/12  08:10:53 
 **/


import javax.swing.*;
import javax.swing.text.html.*;
import java.io.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;


/**A Class representing message displaying utility. 
 * This utility, displays messages, or periodically 
 * checks, while user is logged in, whether one or 
 * more message files (or message directory) have been 
 * modified and display them if they have  
 *
 * <pre>
 * xmotd xm = new xmotd( bitmaplogo,
 *                       paranoid,
 *                       popdown,
 *                       showfilename,
 *                       always,
 *                       stampfile,
 *                       wakeup,
 *                       content,
 *                       files,
 *                       checkdir,
 *                       directory);
 * xm.run();
 * </pre>
 * An instance of <code> xmotd </code> has parameters that 
 * effect its  behavior:<p> 
 * <i> always </i> overrides default 
 * behavior and timestamp is ignored and message(or messages)
 * are always displayed; 
 * <i> wakeup </i> causes xmotd to run on the background 
 * and wakeup periodically every sleep period milliseconds;
 * <i> content </i> is used to specify the type of message
 * (html  or plain or auto); 
 * <i> directory </i> <code> xmotd </code>may be supplied with directory
 * containing files to be displayed;   
 * <i> files </i> one or more files may be displayed;
 * <i> showfilename </i> displays  filename of the file currently
 * being viewed </p>
 * @author Olga Joloud 
 * @version 0.556 April 2001
 *
 **/

public class xmotd implements WindowListener, ActionListener 
{  
 
 
  /** Main is the entry point for the application 
   *  <p>it manages command line parameters, 
   *  instantiate xmotd object and makes call of its run() method </p>
   *  @param arg[] - The command line arguments 
   *         If there is no parameters entered, help message is
   *         printed to the standart output
   **/
  public static void main(String [] arg) 
	throws ClassNotFoundException, FileNotFoundException, InterruptedException
  {
	if (arg.length==0 || (arg[0].toLowerCase().equals("-help")))
	  {
	    String  title = "XMOTD version 0.55\n" +
			"Usage:java xmotd <option> -file <file> [<file>..] | -directory\n" +
			"-bitmaplogo <file>        - name of logo file (in jpeg or gif format)\n"+
			"-paranoid <file>          - name of file with default xmotd message\n"+
			"-showfilename             - show the name of current file\n"+
			"-stampfile <filename>     - the name of the stamp-file\n"+
			"-wakeup <sleep-period>    - run in background mode and check file every \n"+
			"                            <sleep-period> milliseconds\n"+
			"-file <file> [<file> ...] - one or more files to be displayed \n"+
			"-directory <dir>          - check all files in the directory <dir>\n"+
			"-content <type>           - the type of file, where <type> can be one of \n"+
			"                            the following:\n"+
			"                                html - all files have text/html content\n"+
			"                               plain - all files have text/plain content\n"+
			"                                auto - files are html and/or text;\n"+
			"                                       detect content automatically \n";
		

// 	      "-bitmaplogo <file> - name of logo file (in mpeg or gif format)\n"
// 	      + "-paranoid <file> - name of file with default xmotd message\n"
// 	      + "-showfilename - show the name of curent file\n"
// 	      + "-stampfile <stamp-filename> - the name of using stamp-file\n"
// 	      + "-wakeup <sleep-period> - runing in background mode and"
// 	      + "checking file every <sleep-period> milliseconds\n"
// 	      + "-file <file> [<file> ...] - one or more files to be"
// 	      + "displayed may be specified\n"
// 	      + "-directory <dir_name> - checking all files in the directory\n"
// 	      + "-content <type> - the type of files\n"
// 	      + "   <type> has the next values:\n"
// 	      + "   html - all files has text/html content\n"
// 	      + "   plain - all files has text/plain content\n"
// 	      + "   auto - files .html and .htm has text/html"
// 	      + " content, other - text";		
	    System.out.println(title);
	    System.exit(0);
	  }
	// initialize default variables

	String bitmaplogo = "logo.gif";
	String paranoid = null;
	int popdown = 0;
	boolean showfilename = false;
	boolean always = false;
	String stampfile = ".xmotd";
	int wakeup = 0;
	int content = 1;
	String [] files = null;
	boolean checkdir = false;
	String directory = null;

	// reading of command line parameters 	
        // ArrayIndexOutOfBoundException is thrown 
	// if there is no arguments follow parameter

	for(int i=0; i<arg.length; i++)
	  {
		if (arg[i].toLowerCase().equals("-bitmaplogo"))
		  {
		    try
		      {
			
			bitmaplogo = arg[++i];
		      }catch(ArrayIndexOutOfBoundsException e)
			{
			  System.out.println("Use -help");
                          System.exit(0);
			  
			}		      
		      
		      continue;
		  }
		if (arg[i].toLowerCase().equals("-paranoid"))
		  {
			
			try
			  {
			    
			    paranoid = arg[++i];
			  }catch(ArrayIndexOutOfBoundsException e)
			    {
			      System.out.println("Use -help");
			      System.exit(0);
			  
			    }
		      
			  
			  continue;
		  }
		if (arg[i].toLowerCase().equals("-popdown"))
		  {
			popdown = Integer.parseInt(arg[++i]);
			continue;
		  }
		if (arg[i].toLowerCase().equals("-showfilename"))
		  {
			showfilename = true;
			continue;
		  }
		if (arg[i].toLowerCase().equals("-always"))
		  {
			always = true;
			continue;
		  }
			
		if (arg[i].toLowerCase().equals("-stampfile"))
		  {
		    try
		      {
			
			stampfile = arg[++i];
		      }catch(ArrayIndexOutOfBoundsException e)
			{
			  System.out.println("Use -help");
                          System.exit(0);
			  
			}	
		      
		      continue;
		  }
		
		if (arg[i].toLowerCase().equals("-wakeup"))
		  {
		   
		    
		     try
		      {
			wakeup = Integer.parseInt(arg[++i]);
			
		      }catch(NumberFormatException e)
			{
			  System.out.println("Use -help");
                          System.exit(0);
			  
			}	
			continue;
		  }
		// specified to indicate types of message 
		// files

		if (arg[i].toLowerCase().equals("-content"))
		  {
		    try
		      {
			
			if (arg[i+1].toLowerCase().equals("html"))
			  content = 0;
			else
			  if (arg[i+1].toLowerCase().equals("plain"))
			    content = 1;
			  else
			    if (arg[i+1].toLowerCase().equals("auto"))
			      content = 2;
			    else
			      {
				System.out.println("Unknown content type: "+arg[i+1]);
				System.exit(-1);
			      }
		      } catch(ArrayIndexOutOfBoundsException e)
			{
			  System.out.println("Usage -help");
			  System.exit(0);
			}
		      
		      
		      i++;
		      continue;
		  }
		if (arg[i].toLowerCase().equals("-file"))
		  {
			  
		    int j; 
                    int l =0;
		    String [] ft;
		    
		    int count=0;
		    //create temporary array to store file names
   
		    ft= new String[arg.length  - (++i)];
		    for(j=0; i<arg.length; i++)
		      {	 
			
			if (arg[+i].toLowerCase().startsWith("-"))
			  {
			    
			    i=i-1;
                       
			    break;
			  }
			
			ft[j++] = arg[i];
			l++;
		      }
		    files= new String[l];
                    
		    for(j=0; j<l;j++)
		      files[j]=ft[j];
		    
		    
		    continue;		  
		    
		  }
		
		if (arg[i].toLowerCase().equals("-directory"))
		  {
		    checkdir = true;
		    
		   
		    try
		      {
			File f = new File(arg[++i]);
			directory = arg[+i];
			
			
			if (!f.isDirectory())
			  {
			    System.out.println("Invalid directory: "+arg[i]);
			    System.exit(-1);
			  }
		      }catch (ArrayIndexOutOfBoundsException e) 
			{
			  System.out.println("Use -help");
			  System.exit(0);
			}
		      
		      
		  }
	  }
		
	
	// applies run method on the instance of the xmotd class  
		
	(new xmotd(	bitmaplogo,
				paranoid,
				popdown,
				showfilename,
				always,
				stampfile,
				wakeup,
				content,
				files,
				checkdir,
				directory
				)).run();
	   

  } // main ends 
  
  /**<p> Constructor of xmotd  class   
   *  initializes object variables, extracts nesessary history
   *  information: opens input stream for reading serialized object (Hashtable)
   *  reads object from the stream </p>
   *  @param bitmaplogo  name if the logo gif file
   *  @param paranoid  name of the file to be displayed
   *  @param showfilename flag to indicate that the name of the 
   *                      file currently being viewed is displayed
   *  @param always flag that indicates that the message is always 
   *                  displayed
   *  @param wakeup thread sleep period time in milliseconds
   *  @param content type of the files to be displayed; it can be
   *                 <i> plain </i> to display simple text files
   *                 <i> html </i> to display html file
   *                 <i> auto </i> to display both plain and html files
   *  @param files [] string array object  of the file names to be displayed
   *  @param checkdir boolean flag, which when set indicating that the 
   *                  directory specified in String <code> directory </code>
   *                  has to be periodically scanned
   *  @param directory name of the directory to be displayed     
   **/
  
  public xmotd(String bitmaplogo,
	       String paranoid,
	       int popdown,
	       boolean showfilename,
	       boolean always,
	       String stampfile,
	       int wakeup,
	       int content,
	       String [] files,
	       boolean checkdir,
	       String directory) throws ClassNotFoundException, FileNotFoundException
    {
	logo = new ImageIcon(bitmaplogo);
	
	// initialize object member variables
	// this has to be used to disambiguate arguments 
	// from member variable
	
	this.paranoid = paranoid;
	this.popdown = popdown;
	this.showfilename = showfilename;
	this.always = always;
	this.stampfile = stampfile;
	this.wakeup = wakeup;
	this.content = content;
	this.files = files;
	this.checkdir = checkdir;
	this.directory = directory;
	if (files == null && directory == null)
	  {
	    System.out.println("Please enter file or directory");
            System.exit(0);
	    
	  }
	
	try{
	  // Vector is created to store the information 
	  // regarding the files 

	  fileholder = new Vector();

          //Opens Input stream for reading serilized object
   
	  ObjectInputStream ois = 
		new ObjectInputStream(new FileInputStream(stampfile));

	  //sterilized object is extracted

	  times = (Hashtable)ois.readObject();
	}catch(IOException x)
	  {
		times = new Hashtable();
		System.out.println("Warning: stampfile not found. Using new values");
	  }
	
	createFrame();
  }
 
  /// Parameters
  ImageIcon logo;
  String paranoid;
  String bitmaplogo;
  int popdown;
  boolean showfilename;
  boolean always;
  String stampfile;
  Hashtable times;
  boolean tail;
  int wakeup;
  int content;
  String [] files;
  Vector fileholder;
  boolean checkdir;
  String directory;
  
  
  /// Interface elements
  JFrame frame;
  JEditorPane fileArea;
  JLabel fName=null;
  JLabel fDate=null;
  JButton button;
  JScrollPane scrollPane;  
  

  int firstMsg = 0; // message currently displayed  
  int lastMsg = 0;  // last message in the vector waiting to be shown

  /** Creates GUI.Extends  JFrame to show HTML files.
   * <p> Frame contains panels in them file 
   * name <i>label</i>, date <i> label</i>, <i>button,</i>
   * <i> icon.</i>
   * Adds <code>actionListener</code> to the button, so that when user 
   * clicks on the button another file will be shown. </p>
   **/
  public void createFrame()
  {
	frame = new JFrame("Message Of The Day");
	frame.addWindowListener(this);
	frame.setSize(new Dimension(600,600));













	frame.setVisible(false);
	frame.getContentPane().setLayout(new BorderLayout());
	
	JPanel p1 = new JPanel();
	p1.setLayout(new BorderLayout());
	
	JPanel p2 = new JPanel();
	if (showfilename)
	  p2.setLayout(new GridLayout(3, 1));
	else
	  p2.setLayout(new GridLayout(2, 1));
	
	if (showfilename)
	  {
		JPanel p21 = new JPanel();
		p21.add(new JLabel("File:"));
		fName = new JLabel("text.txt");
		fName.setForeground(Color.black);
		p21.add(fName);
		p2.add(p21);
	  }
	
	JPanel p22 = new JPanel();
	p22.add(new JLabel("Date:"));
	fDate = new JLabel("datetext");
	fDate.setForeground(Color.black);
	p22.add(fDate);
	p2.add(p22);
	
	JPanel p23 = new JPanel();
	button = new JButton("Next");

	// Add the action listener to button so that when
	// user presses the Next button
	// new file is shown
	
	button.addActionListener(this);
	p23.add(button);
	p2.add(p23);
	
	JLabel l1 = new JLabel();
	l1.setIcon(logo);
	p1.add(l1,"West");
        
	p1.add(p2,"Center");
	
	fileArea = new JEditorPane();
	
	// Make JEditorPane non-editable
	// By default, JEditorPane is editable and works as 
	// HTML editor not as a browser. To make it work as browser it
	// has to be non-editable
     
	fileArea.setEditable(false);
	
	scrollPane = new JScrollPane(fileArea);

	frame.getContentPane().add(p1,"North");
    // add scrollpane to the frame
	frame.getContentPane().add(scrollPane,"Center");
  }
  /**Implements file timestamp checking procedure 
   * Constantly lists files in the directory and adds them to
   * the String array object. Checks if the file was modified, 
   * by comparing current timestamp of the file with the 
   * last saved one in the Hashtable. 
   * Also display of the firest message is done here.
   * Following messages are displayed in the <code> actionPerformed </code>
   **/
   
  public void run() throws InterruptedException
  {
	do{
	  if (checkdir)
		{
		  // Create a File object to represent a file in that directory
		  File f = new File(directory);

		  // list files in the directory

		  String entries[] = f.list();
              
		  files = new String[ entries.length];
			  
		  String separator = System.getProperty("file.separator");
		  for (int j =0; j< entries.length; j++)
			{
				  
			  if( directory.endsWith(separator)){
				files[j] = directory + entries[j];
			  }
			  else { 
				// add separator if initial directory
				// does not have it
				files[j] = directory + separator + entries[j];
			  }
			}
			  
		}
		  
	  // if special warning message is specified
	  // add only this message to the String array 
	  // of files and set flag always to true 
	  // that the timestamp would be ignored

	  if (paranoid != null)
		{
		  firstMsg = 0;
		  lastMsg = 0;
			  
		  files = new String[ 1];
		  files[0] = paranoid;
		  always = true;
			  
		}
		  
	  for(int i=0; i<files.length; i++)
		{
			  
			  
		  File f = new File(files[i]);
                
		  // find when file was last modified
		  long l1 = f.lastModified();
			  
		  // get files timestamp from Hashtable
		    
		  Long ll2 = (Long)times.get(files[i]);
		  long l2=0;
		  if (ll2!=null)
			l2 = ll2.longValue();

		  //if the file was modified or option always is specified

		  if ((l2!=l1) || always)
			{
			  try{

				fileChanged(files[i], l1);
				frame.setVisible(true);
			  }catch(IOException x)
				{
				  System.out.println(x.getMessage());
                                  
				  
				}
			}
		}
		  
		  
	  try{  
		if (firstMsg==0) //if none of the messages have been shown
		  {
				
			if (lastMsg!=0) //if there are messages 
			  {
				firstMsg=1;
				// showing first message 
				showFile();
			  }
                				
		  }
		else
		  {
			// if first message  have been shown
			// set button to next if there are mo messages to be 
			// shown
			if (firstMsg!=lastMsg) button.setText("Next");
			else button.setText("Dismiss");
		  }
	  }catch(IOException x)
		{
		  System.out.println(x.getMessage());
		}
		  
	  if (wakeup!=0) Thread.currentThread().sleep(wakeup);
		  
	}while(wakeup!=0);
		
	if (lastMsg==0)
	  System.exit(0);
  }
    /** Creates hystory regarding message file  
     * Creates <code> HistoryItem </code> object, adds it to <code>
     *  Hashtable </code> object, Serializes <code>Hashtable </code>
     *  object and saves it to the file. 
     *@param fileName  name of the file to be displayed
     *@param mTime  time the file was last modified
     **/
  
  public void fileChanged(String fileName, long mTime) throws IOException
    {
	lastMsg++;
		
	HistoryItem hi = new HistoryItem(fileName, mTime);
	   
	fileholder.addElement(hi);
	
	times.put(fileName, new Long(mTime));

	//serilize the object 
	ObjectOutputStream oos = 
	  new ObjectOutputStream( new FileOutputStream(stampfile));
        
	oos.reset();
	oos.writeObject(times);
	oos.close();
  }

  /**Performs message reading 
   * Reads message from the file to the string and loads it to JEditorPane   
   * <p>HistoryItem object is read from vector, <i>file name</i> is extracted
   * by accessing public member <code>name</code> from <code>HistoryItem</code>
   * object. ContetType of <code> JEditorPane </code> depending on type of 
   * the message file(html/plain) </p>   
   **/
      
  void showFile() throws IOException
  {
	// returns object at (firstMsg -1), because vector starts count
	// of elements from 0
	
	HistoryItem hi  = (HistoryItem)fileholder.elementAt(firstMsg-1);
        //System.out.println(hi);
	
	String fileName = hi.name;
	
	BufferedReader in = new BufferedReader(new FileReader(fileName));
	
	if (fName!=null)
	  fName.setText(fileName);
	fDate.setText(hi.getDate());
	
	String text = "";
	String s;
	while((s = in.readLine())!=null) //read line check for end of file
	  text+="\r\n" + s;
	in.close();  //close stream
	
	fileArea.setText("");
	if (content==0) fileArea.setContentType("text/html");
	if (content==1) fileArea.setContentType("text/plain");
	if (content==2)
	  {
		if (fileName.toLowerCase().endsWith(".html") 						  
			|| fileName.toLowerCase().endsWith(".htm")) {		  
		  fileArea.setContentType("text/html");
		} else {		  
		  fileArea.setContentType("text/plain");
		}		
	  }
	fileArea.setText(text);

	if (firstMsg!=lastMsg) button.setText("Next");
	else button.setText("Dismiss");
	
	scrollPane.getVerticalScrollBar().setValue(0);
	frame.show();
  
  }
   
  public void windowOpened(WindowEvent e){}
  public void windowClosing(WindowEvent e){System.exit(0);}
  public void windowClosed(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowActivated(WindowEvent e){}
  public void windowDeactivated(WindowEvent e){}
  
  /** ActionListener event handler method 
   * Displays new messages in the text display window 
   * synchronized with message files using two
   * global variables <i> lastMsg </i> and <i> firstMsg </i>
   * @param e  The event describing what action occured
   * 
   **/
  public void actionPerformed(ActionEvent e)
  {
	// if there is more messages 
	if (firstMsg != lastMsg)
	  {
		firstMsg++;
		 
		try{
		  // display message
		  showFile();
		}catch(IOException x)
		  {
			System.out.println(x.getMessage());
		  }
	  }
	else
	  {
		if (wakeup==0)
		  System.exit(0);
		else
		  {
			  
			firstMsg =0;
			lastMsg = 0;
			fileholder.removeAllElements();
			frame.setVisible(false);
		  }
	  }
  }
} // end of class xmotd
 




